/*
 * Chron
 * Copyright Jonathan Booth, Oct 13, 1996
 *
 * Last Update: Oct 13, 1996
 *
 * A rewrite of the X-windows clock 'Chron', except prettier.
 */

#include "Application.h"

Application::Application()
	: BApplication(APP_SIG)
{
	win = NULL;
}

Application::~Application()
{
	BApplication::~BApplication();
}

bool Application::QuitRequested()
{
	WritePref();
	
	if (win) {
		BMessage *msg = new BMessage(B_QUIT_REQUESTED);
		PostMessage(msg,win);
	}
	
	return TRUE;
}

void Application::ReadyToRun()
{
	BApplication::ReadyToRun();
	
	BVolume *Bootdisk = new BVolume(0);
	Settings = new BDirectory();
	BDirectory *tmp = new BDirectory();
	File = new BFile();

	Bootdisk->GetRootDirectory(Settings);
	
	if (Settings->Contains("system")) {
		Settings->GetDirectory("system",tmp);
		if (tmp->Contains("settings")) {
			tmp->GetDirectory("settings",Settings);
			if (Settings->Contains("Chron_settings")) {
				Settings->GetFile("Chron_settings",File);
			} else {
				// Create the file, and write some data there
				Settings->Create("Chron_settings",File);

				// Find the current screen size
				screen_info cur;
				get_screen_info(&cur);

				// Dimensions and offset
				LenX = 110;
				LenY = 110;
				OffsetX = cur.frame.right - LenX - 5;
				OffsetY = cur.frame.bottom - LenY - 5;
				
				WritePref();
			}
		}
	}

	// Put up the main menu
	main = MakeMainMenu();
	SetMainMenu(main);
	
	ReadPref();

	BRect area(OffsetX,OffsetY,LenX + OffsetX,
			   LenY + OffsetY);
	SecStat = TRUE;
	win = new ClockWindow(area);
}

void Application::AboutRequested()
{
	BAlert *about = new BAlert("","Chron, Copyright October 21, 1996\n"
							   "Jonathan Booth (kamikaze@imsa.edu)\n\n"
							   "If you use Chron, please send an e-mail\n"
							   "to me. If you feel generous and wish to\n"
							   "make a contribution to the author\n"
							   "(#include PoorStarvingCollegeStudent.h)\n"
							   "send it to:\n\n"
							   "   Jonathan Booth\n"
							   "   29W131 Batavia Road\n"
							   "   Warrenville IL, 60555\n\n",
							   "Ok!");
	about->Go();
}

void Application::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case APP_SEC_CHANGE:
			if (win) {
				SecStat = !SecStat;
				win->ChangeSec();
			}
			break;
		case WIN_REMOVED:
			if ((ClockWindow *)msg->FindObject("window") == win) {
				BMessage *bye = new BMessage(B_QUIT_REQUESTED);
				PostMessage(bye);
			}
		case WIN_MOVED:
			if (msg->FindFloat("new_x") != 0) {
				OffsetX = msg->FindFloat("new_x");
			}
			if (msg->FindFloat("new_y") != 0) {
				OffsetY = msg->FindFloat("new_y");
			}
			
			break;
		case WIN_RESIZED:
			LenX = msg->FindFloat("new_w");
			LenY = msg->FindFloat("new_h");
			
			break;
		default:
			BApplication::MessageReceived(msg);	
	}	
}

BPopUpMenu * Application::MakeMainMenu()
{
	// Definitions
	BMenuItem *item;
	BSeparatorItem *sep;
	BPopUpMenu *ret = new BPopUpMenu("",FALSE,FALSE);
	
	// About entry
	item = new BMenuItem("About Chron...",
						 new BMessage(B_ABOUT_REQUESTED),
						 'A');
	item->SetTarget(be_app);
	ret->AddItem(item);
	
	// Seperator
	sep = new BSeparatorItem;
	ret->AddItem(sep);
	
	// Seconds?
	item = new BMenuItem("Swap Seconds",
						 new BMessage(APP_SEC_CHANGE),
						 'S');
	item->SetTarget(be_app);
	ret->AddItem(item);
	
	// Seperator
	sep = new BSeparatorItem;
	ret->AddItem(sep);
	
	// Quit
	item = new BMenuItem("Quit",
						 new BMessage(B_QUIT_REQUESTED),
						 'Q');
	item->SetTarget(be_app);
	ret->AddItem(item);
	
	return ret;
}

void Application::ReadPref()
{
	File->Open(B_READ_ONLY);
//	File->Seek(0,B_SEEK_TOP);

	File->Read(&OffsetX,2);
	File->Read(&OffsetY,2);
	File->Read(&LenX,2);
	File->Read(&LenY,2);
	
	File->Close();
}

void Application::WritePref()
{
	File->Open(B_READ_WRITE);
//	File->Seek(0,B_SEEK_TOP);

	File->Write(&OffsetX,2);
	File->Write(&OffsetY,2);
	File->Write(&LenX,2);
	File->Write(&LenY,2);
			
	File->Close();
}